/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMMasterViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//



#import "SMMasterViewController.h"
#import "SMDetailViewController.h"

@interface SMMasterViewController () {
    NSMutableArray *_objects;
}

@end

@implementation SMMasterViewController



- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = @"Grocery List";
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            self.clearsSelectionOnViewWillAppear = NO;
            self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
        }
    }
    return self;
}
						
- (void)viewDidLoad
{
    [super viewDidLoad];
    NSUbiquitousKeyValueStore* store = [NSUbiquitousKeyValueStore defaultStore];
        
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(kvsDidChange:)
     name:NSUbiquitousKeyValueStoreDidChangeExternallyNotification
     object:nil];
        
    [store synchronize];
    NSDictionary *d = [store dictionaryRepresentation];
    NSLog(@"d is %@", d);
    _objects = [NSMutableArray arrayWithArray:[d allKeys]];
    
    UIBarButtonItem *addButton =
        [[UIBarButtonItem alloc]
         initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                              target:self
                              action:@selector(insertNewObject:)];
    
    self.navigationItem.rightBarButtonItem = addButton;
}

- (void) kvsDidChange:(NSNotification *) notification {
    NSDictionary *userInfo = [notification userInfo];
    NSLog(@"userInfo %@", userInfo)
	
    NSArray *changedKeys =
        [userInfo objectForKey:NSUbiquitousKeyValueStoreChangedKeysKey];
    
    NSLog(@"kvs changed remotely %@", changedKeys);
    NSUbiquitousKeyValueStore* store = [NSUbiquitousKeyValueStore defaultStore];
    NSDictionary *d = [store dictionaryRepresentation];
    NSLog(@"d is %@", d);
    
    _objects = [NSMutableArray arrayWithArray:[d allKeys]];
    [self.tableView reloadData];   
}

- (void)insertNewObject:(id)sender {
    
    NSUUID *itemId = [NSUUID UUID];
    NSString *itemIdString = [itemId UUIDString];
    
    NSUbiquitousKeyValueStore* store = [NSUbiquitousKeyValueStore defaultStore];
    [store setString:@"Untitled" forKey:itemIdString];
    [store synchronize];
    
    NSDictionary *d = [store dictionaryRepresentation];
    _objects = [NSMutableArray arrayWithArray:[d allKeys]];    
    [self.tableView reloadData];
    
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}



#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objects.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    NSString *itemKey = [_objects objectAtIndex:indexPath.row];
    NSUbiquitousKeyValueStore* store = 
		[NSUbiquitousKeyValueStore defaultStore];
    NSString *item = [store objectForKey:itemKey];
    
    UITableViewCell *cell =
        [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] 
				initWithStyle:UITableViewCellStyleSubtitle
              reuseIdentifier:CellIdentifier];
        if ([[UIDevice currentDevice] userInterfaceIdiom] ==
                UIUserInterfaceIdiomPhone) {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
    }
    
    cell.textLabel.text = item;
    cell.detailTextLabel.text = itemKey;

    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [_objects removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSDate *object = [_objects objectAtIndex:indexPath.row];
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
	    if (!self.detailViewController) {
	        self.detailViewController = [[SMDetailViewController alloc] initWithNibName:@"SMDetailViewController_iPhone" bundle:nil];
            self.detailViewController.detailItem = object;
	    }
        [self.navigationController pushViewController:self.detailViewController animated:YES];
    } else {
            self.detailViewController.detailItem = object;
    }
}

@end
