/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMGroceryDocument.m
//  Grocery
//
//  Created by Cesare Rocchi on 5/18/13.
//
//

#import "SMGroceryDocument.h"


#define kItemsKey @"items"
#define kItemsWrapperKey @"items.wra"

NSString *const SMItemModifiedNotification =
    @"com.studiomagnolia.itemModified";
NSString *const SMModificationSavedNotification =
    @"com.studiomagnolia.modificationSaved";


@implementation SMGroceryDocument

- (id)initWithFileURL:(NSURL *)url {  
    if ((self = [super initWithFileURL:url])) {
        _items = [[NSMutableArray alloc] init];
        
        [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(itemModified:)
                       name:SMItemModifiedNotification
                    object:nil];
         }
         return self;
}

- (void) itemModified:(NSNotification *) notification {
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
  
      if (success) {
          [[NSNotificationCenter defaultCenter]
              postNotificationName:SMModificationSavedNotification
                            object:nil];
      }
  }];
}

- (id)contentsForType:(NSString *)typeName error:(NSError **)outError {
    
    NSMutableDictionary *wrappers = [NSMutableDictionary dictionary];
    // create buffer of data
    NSMutableData *data = [NSMutableData data];
    // initialized archiver
    NSKeyedArchiver *arch =
    [[NSKeyedArchiver alloc] initForWritingWithMutableData:data]; // encoding
    [arch encodeObject:_items forKey:kItemsKey];
    [arch finishEncoding];
    NSFileWrapper *entriesWrapper =
    [[NSFileWrapper alloc] initRegularFileWithContents:data];
    [wrappers setObject:entriesWrapper
                 forKey:kItemsWrapperKey];
    // here you could add another wrapper for other resources,
    // like images
    NSFileWrapper *rootWrapper =
    [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
    
    return rootWrapper;
}

- (BOOL)loadFromContents:(id)contents
                  ofType:(NSString *)typeName
                   error:(NSError **)outError {
    
    NSFileWrapper *rootWrapper = (NSFileWrapper *)contents;
    NSDictionary *children = [rootWrapper fileWrappers];
    NSFileWrapper *itemsWrapper = [children objectForKey:kItemsWrapperKey];
    NSData *data = [itemsWrapper regularFileContents];
    NSKeyedUnarchiver *arch = [[NSKeyedUnarchiver alloc]
                               initForReadingWithData:data];
    _items = [arch decodeObjectForKey:kItemsKey];
    
    return YES;
}

- (void) addItem:(SMGroceryItem *) groceryitem {
    [_items addObject:groceryitem];
    
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
      if (success) {
          NSLog(@"new item added");
      }
  }];
}

- (void) removeItem:(SMGroceryItem *) groceryitem {
    [_items removeObject:groceryitem];
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
      if (success) {
          NSLog(@"item deleted");
      }
  }];
}

- (SMGroceryItem *)entryAtIndex:(NSUInteger)index { 
    if (index < _items.count) {  
        return [_items objectAtIndex:index]; 
    } else { 
        return nil; 
    }
}
- (NSInteger ) count {
    return self.items.count;
}


@end
