/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMMasterViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SMMasterViewController.h"
#import "SMDetailViewController.h"
#define kFILENAME @"grocery.gro"
@interface SMMasterViewController () {
    NSMutableArray *_objects;
    NSMetadataQuery *_query;
}

- (void) loadData:(NSMetadataQuery *)query;

@end

@implementation SMMasterViewController


- (void) loadGrocery {
    _query = [[NSMetadataQuery alloc] init];
    [_query setSearchScopes:[NSArray arrayWithObject:
                             NSMetadataQueryUbiquitousDocumentsScope]];
    
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"%K == %@",
                         NSMetadataItemFSNameKey,
                         kFILENAME];
    [_query setPredicate:pred];
   
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(queryDidFinishGathering:)
     name:NSMetadataQueryDidFinishGatheringNotification
     object:_query];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(queryDidFinishUpdating:)
     name:NSMetadataQueryDidUpdateNotification
     object:_query];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(modificationSaved)
     name:SMModificationSavedNotification
     object:nil];
    
    [_query startQuery]; 
}

- (void) modificationSaved { [_query enableUpdates];
    [self.tableView reloadData];
}

- (void) queryDidFinishUpdating:(NSNotification *) notification {

    NSLog(@"queryDidFinishUpdating");
    NSMetadataQuery *q = [notification object];
    [_query stopQuery];
    [self loadData:q];
    [_query startQuery];
    
}

- (void) queryDidFinishGathering:(NSNotification *) notification {
    
    NSLog(@"queryDidFinishGathering");
    [_query stopQuery];
    
    [[NSNotificationCenter defaultCenter] 
      removeObserver:self 
                name:NSMetadataQueryDidFinishGatheringNotification 
              object:_query];
        
    [self loadData:_query];
    
    [_query startQuery];
    
}

- (void) loadData:(NSMetadataQuery *)query {
    if ([query resultCount] == 1) {
        NSMetadataItem *item = [query resultAtIndex:0];
        NSURL *url = [item valueForAttribute:NSMetadataItemURLKey];

        if (!self.itemsDocument) {
            self.itemsDocument = [[SMGroceryDocument alloc] initWithFileURL:url];
        }
        [self.itemsDocument openWithCompletionHandler:^ (BOOL success) {
            if (success) {
            NSLog(@"loadData - doc opened from cloud %i", self.itemsDocument.count);
            [self.tableView reloadData];   
            [self.itemsDocument closeWithCompletionHandler:^(BOOL success) { 
                NSLog(@"doc closed");
                }
             ];  
        } else { 
            NSLog(@"failed to open"); 
        } }]; 
        
    } else { // No notes in iCloud
        NSURL *ubiqContainer = [[NSFileManager defaultManager]
                                URLForUbiquityContainerIdentifier:nil];
        NSURL *ubiquitousPackage = [[ubiqContainer
                                     URLByAppendingPathComponent:@"Documents"]
                                     URLByAppendingPathComponent:kFILENAME];
        SMGroceryDocument *doc = [[SMGroceryDocument alloc]
                                  initWithFileURL:ubiquitousPackage];
        self.itemsDocument = doc;
        
        [doc saveToURL:[doc fileURL]
      forSaveOperation:UIDocumentSaveForCreating
     completionHandler:
         ^(BOOL success) {
             NSLog(@"new document saved to iCloud"); 
         }];
    }
}


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = @"Grocery list";
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            self.clearsSelectionOnViewWillAppear = NO;
            self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
        }
    }
    return self;
}
							
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _objects = [[NSMutableArray alloc] init];

    self.navigationItem.leftBarButtonItem = self.editButtonItem;

    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                                                                               target:self
                                                                               action:@selector(insertNewObject:)];
    self.navigationItem.rightBarButtonItem = addButton;
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(noteUpdated:)
                                                 name:groceryItemUpdated
                                               object:nil];
    
    [self loadGrocery];
    
}


- (void) noteUpdated:(id)sender {

    SMGroceryItem *i = (SMGroceryItem*)sender;

    NSUInteger index = [_objects indexOfObject:i];
    
    NSIndexPath *a = [NSIndexPath indexPathForItem:index
                                         inSection:0];
    
    //just reload reload that item
    [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:a]
                          withRowAnimation:UITableViewRowAnimationNone];
    
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (void)insertNewObject:(id)sender {
    SMGroceryItem *groceryItem = [[SMGroceryItem alloc] init];
    groceryItem.itemName = @"Untitled";
    [self.itemsDocument addItem:groceryItem];
    [self.tableView reloadData];
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView
 numberOfRowsInSection:(NSInteger)section {
    return self.itemsDocument.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
		 
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView
                        dequeueReusableCellWithIdentifier:CellIdentifier];
						
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                      reuseIdentifier:CellIdentifier];
        if ([[UIDevice currentDevice] userInterfaceIdiom] ==
                UIUserInterfaceIdiomPhone) {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;  
        }
    }
	
    SMGroceryItem *groceryItem = [self.itemsDocument entryAtIndex:indexPath.row];
    cell.textLabel.text = groceryItem.itemName;
    return cell;  
}

- (BOOL)tableView:(UITableView *)tableView
canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return NO;
}

- (void)tableView:(UITableView *)tableView
didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    SMGroceryItem *object = [self.itemsDocument entryAtIndex:indexPath.row];
    if ([[UIDevice currentDevice] userInterfaceIdiom] ==
        UIUserInterfaceIdiomPhone) {
        self.detailViewController = [[SMDetailViewController alloc]
                                    initWithNibName:@"SMDetailViewController_iPhone"
                                             bundle:nil];
											 
        self.detailViewController.detailItem = object;
        [self.navigationController pushViewController:self.detailViewController
                                             animated:YES];
    } else {
        self.detailViewController.detailItem = object;   
    }
    [_query disableUpdates];
}
- (void)tableView:(UITableView *)tableView
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
forRowAtIndexPath:(NSIndexPath *)indexPath {

    if (editingStyle == UITableViewCellEditingStyleDelete) { 
        SMGroceryItem *i = [self.itemsDocument entryAtIndex:indexPath.row];
        if (self.detailViewController.detailItem == i) {
            self.detailViewController.detailItem = nil;    
        }
        [self.itemsDocument removeItem:i];  
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
                         withRowAnimation:UITableViewRowAnimationFade];
    }
}
@end
