/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMDetailViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SMDetailViewController.h"
#import "SMGroceryDocument.h"

@interface SMDetailViewController ()
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation SMDetailViewController


#pragma mark - Managing the detail item

- (void) setSelectedItemIndex:(NSInteger)newItemIndex {
    
    if (_selectedItemIndex != newItemIndex) {
        _selectedItemIndex = newItemIndex;
    }
    
    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }
    
    [self configureView];
}

- (void)configureView {
    self.itemNameField.text =
        [[self.itemsDocument itemAtIndex:self.selectedItemIndex] itemName];
    UIImage *i = [self.itemsDocument imageAtIndex:self.selectedItemIndex];
    self.itemImageView.image = i;
}

- (void)viewDidLoad
{
    [super viewDidLoad];  
	
    UIBarButtonItem *saveButton =
     [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSave
                                                   target:self
                                                   action:@selector(saveItem:)];
    
    self.navigationItem.rightBarButtonItem = saveButton;
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] ==
        UIUserInterfaceIdiomPhone) {
        self.itemNameField = [[UITextField alloc]
                              initWithFrame:CGRectMake(20, 20, 280, 31)]; 
							  
    } else {     
        self.itemNameField = [[UITextField alloc]
                              initWithFrame:CGRectMake(20, 20, 340, 31)];     
    }
    
    self.itemNameField.borderStyle = UITextBorderStyleRoundedRect;
    self.itemNameField.clearButtonMode = UITextFieldViewModeWhileEditing;
    self.itemNameField.delegate = self;
	
    [self.view addSubview:self.itemNameField];
    
    self.itemImageView = [[UIImageView alloc] initWithFrame:
                          CGRectMake(20, 60, 50, 50)];
						  
    self.itemImageView.userInteractionEnabled = YES;
    self.itemImageView.backgroundColor = [UIColor redColor];
    [self.view addSubview:self.itemImageView];
    
    UITapGestureRecognizer *tapRecognizer =
      [[UITapGestureRecognizer alloc] initWithTarget:self
                                              action:@selector(showImagePicker:)];
											  
    tapRecognizer.numberOfTapsRequired = 1;
    [self.itemImageView addGestureRecognizer:tapRecognizer]; 
    [self configureView];
	
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {

    [[NSNotificationCenter defaultCenter]
        removeObserver:self
                  name:UIDocumentStateChangedNotification
                object:nil];
    
    
}

- (void) itemHasChanged:(id) sender {  
    [self configureView];
}


- (void) saveItem:(id) sender {
    [[NSNotificationCenter defaultCenter]
     postNotificationName:SMItemModifiedNotification
                   object:@[[NSNumber numberWithInteger:self.selectedItemIndex],
                            self.itemNameField.text,
                            self.itemImageView.image]];
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] ==
        UIUserInterfaceIdiomPhone) {
        [self.navigationController popViewControllerAnimated:YES]; 
    } else {  
        [self.itemNameField resignFirstResponder];
        self.itemNameField.text = @""; 
    }
}

- (void) showImagePicker:(id)sender {
    UIImagePickerController *picker = [[UIImagePickerController alloc] init];
    picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    picker.delegate = self;
    
    if (([[UIDevice currentDevice] userInterfaceIdiom] ==
         UIUserInterfaceIdiomPad)) {
        self.popoverPickerController = [[UIPopoverController alloc]
                                        initWithContentViewController:picker];
        
        [self.popoverPickerController
         presentPopoverFromRect:self.itemImageView.frame
                         inView:self.view
       permittedArrowDirections:UIPopoverArrowDirectionAny
                       animated:YES]; 
					   
    } else {
        [self presentViewController:picker
                           animated:YES
                         completion:nil];  
    }
}

- (void)imagePickerController:(UIImagePickerController *)picker
didFinishPickingMediaWithInfo:(NSDictionary *)info {

    self.itemImageView.image =
    [info objectForKey:@"UIImagePickerControllerOriginalImage"];
    
    if (([[UIDevice currentDevice] userInterfaceIdiom] ==
         UIUserInterfaceIdiomPad)) {
        
        [self.popoverPickerController dismissPopoverAnimated:YES];
        
    } else {
        [self dismissViewControllerAnimated:YES
                                 completion:nil];
    }
}
    


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = NSLocalizedString(@"Item Detail", @"Detail");
    }
    return self;
}
							
#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController
     willHideViewController:(UIViewController *)viewController
          withBarButtonItem:(UIBarButtonItem *)barButtonItem
       forPopoverController:(UIPopoverController *)popoverController {
    
    barButtonItem.title = @"Grocery list";
    [self.navigationItem setLeftBarButtonItem:barButtonItem
                                     animated:YES];
    self.masterPopoverController = popoverController;
    
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
