/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMGroceryDocument.m
//  Grocery
//
//  Created by Cesare Rocchi on 5/18/13.
//
//

#import "SMGroceryDocument.h"

#define kItemsKey @"items"
#define kImagesKey @"images"
#define kItemsWrapperKey @"items.wra" 
#define kImagesWrapperKey @"images.wra"

NSString *const SMItemModifiedNotification =
    @"com.studiomagnolia.itemModified";
NSString *const SMModificationSavedNotification =
    @"com.studiomagnolia.modificationSaved";

@implementation SMGroceryDocument

- (id)initWithFileURL:(NSURL *)url {
    if ((self = [super initWithFileURL:url])) {
        _items = [[NSMutableArray alloc] init];
        _images = [[NSMutableArray alloc] init];
        
        [[NSNotificationCenter defaultCenter]
           addObserver:self
              selector:@selector(itemModified:)
                  name:SMItemModifiedNotification
                object:nil];
    }
    return self; 
}

- (void) itemModified:(NSNotification *) notification {
    NSArray *a = [notification object];
    SMGroceryItem *item = [_items objectAtIndex:[[a objectAtIndex:0]
                                                 intValue]];

    item.itemName = [a objectAtIndex:1];
    UIImage *image = [a objectAtIndex:2];
    [_images setObject:image atIndexedSubscript:[[a objectAtIndex:0]
                                                 intValue]]; 
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
      
         if (success) {        
             [[NSNotificationCenter defaultCenter]
              postNotificationName:SMModificationSavedNotification
              object:nil];      
         }
  }];  
}


- (id)contentsForType:(NSString *)typeName error:(NSError **)outError {
    NSMutableDictionary *wrappers = [NSMutableDictionary dictionary];
    //Encoding items
    NSMutableData *data = [NSMutableData data];
    NSKeyedArchiver *arch =
    [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
    [arch encodeObject:_items forKey:kItemsKey];
    [arch finishEncoding];
    NSFileWrapper *entriesWrapper = [[NSFileWrapper alloc]
                                     initRegularFileWithContents:data];
    [wrappers setObject:entriesWrapper forKey:kItemsWrapperKey];
    //Encoding images
    NSMutableData *imageData = [NSMutableData data];
    NSKeyedArchiver *imagesArchiver =
    [[NSKeyedArchiver alloc] initForWritingWithMutableData:imageData];
    [imagesArchiver encodeObject:_images forKey:kImagesKey];
    [imagesArchiver finishEncoding];
    NSFileWrapper *imagesWrapper = [[NSFileWrapper alloc]
                                    initRegularFileWithContents:imageData];
    [wrappers setObject:imagesWrapper forKey:kImagesWrapperKey];
    
    NSLog(@"saving: images are %@", _images); NSFileWrapper *rootWrapper = 
    [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
    return rootWrapper;
}


- (BOOL)loadFromContents:(id)contents
                  ofType:(NSString *)typeName
                   error:(NSError **)outError {
    NSFileWrapper *rootWrapper = (NSFileWrapper *)contents;
    NSDictionary *children = [rootWrapper fileWrappers];
    
    NSFileWrapper *itemsWrapper = [children objectForKey:kItemsWrapperKey];
    NSData *data = [itemsWrapper regularFileContents];
    NSKeyedUnarchiver *arch = [[NSKeyedUnarchiver alloc]
                               initForReadingWithData:data];
    _items = [arch decodeObjectForKey:kItemsKey];
    
    NSFileWrapper *imagesWrapper = [children objectForKey:kImagesWrapperKey];
    NSData *imagesData = [imagesWrapper regularFileContents];
    NSKeyedUnarchiver *imagesUnarchiver = [[NSKeyedUnarchiver alloc]
                                           initForReadingWithData:imagesData];
    _images = [imagesUnarchiver decodeObjectForKey:kImagesKey];
    NSLog(@"loading: images are %@", _images);
    return YES;
}


- (void) addItem:(SMGroceryItem *) groceryitem {
    [_items addObject:groceryitem];
    [_images addObject:[UIImage imageNamed:@"placeholder.png"]];
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
      if (success) {
          NSLog(@"new item added");
      }
  }]; 
}

- (void) removeItemAtIndex:(NSInteger) index {
    [_items removeObjectAtIndex:index];
    [_images removeObjectAtIndex:index];
    [self saveToURL:[self fileURL]
   forSaveOperation:UIDocumentSaveForOverwriting
  completionHandler:^(BOOL success) {
     if (success) {
         NSLog(@"item deleted");
     }   
  }]; 
}

- (SMGroceryItem *)itemAtIndex:(NSUInteger)index {
    if (index < _items.count) {
        return [_items objectAtIndex:index];
    } else {
        return nil;
    }
}

- (UIImage *)imageAtIndex:(NSUInteger)index { 
    if (index < _images.count) {    
        return [_images objectAtIndex:index];      
    } else {     
        return nil;      
    }   
}

- (NSInteger ) count {  
    return self.items.count;   
}

@end
