/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMMasterViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SMMasterViewController.h"
#import "SMDetailViewController.h"

@interface SMMasterViewController () {
    NSMutableArray *_objects;
    NSMetadataQuery *_query;
}

- (void) loadData:(NSMetadataQuery *)query;

@end

@implementation SMMasterViewController


- (void) loadGrocery {
    _query = [[NSMetadataQuery alloc] init];
    [_query setSearchScopes:[NSArray arrayWithObject:
                             NSMetadataQueryUbiquitousDocumentsScope]]; 
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"%K like 'Item_*'",
                         NSMetadataItemFSNameKey];
    [_query setPredicate:pred];
    
    [[NSNotificationCenter defaultCenter]
      addObserver:self
         selector:@selector(queryDidFinishGathering:)
             name:NSMetadataQueryDidFinishGatheringNotification
           object:_query];
    [[NSNotificationCenter defaultCenter]
      addObserver:self
         selector:@selector(documentHasChangedState:)
             name:UIDocumentStateChangedNotification
           object:nil];
    [_query startQuery]; 
}

- (void)documentHasChangedState:(id)sender {
    [self.tableView reloadData];  
}


- (void) queryDidFinishGathering:(NSNotification *) notification {
    
    NSLog(@"queryDidFinishGathering");
    [_query stopQuery];
    
    [[NSNotificationCenter defaultCenter] 
     removeObserver:self 
     name:NSMetadataQueryDidFinishGatheringNotification 
     object:_query];
        
    [self loadData:_query];
        
}

- (void) loadData:(NSMetadataQuery *)query {
    
    [_objects removeAllObjects];
    
    NSArray *res = [query results];
    
    for (NSMetadataItem *item in res) {
        
        NSURL *url = [item valueForAttribute:NSMetadataItemURLKey];
        SMGroceryItem *groceryItem = [[SMGroceryItem alloc] initWithFileURL:url];
                
        [_objects addObject:groceryItem];
        
        [groceryItem openWithCompletionHandler:^(BOOL success) {
            if (success) {
                
                [self.tableView reloadData];

                
            } else {
                
                NSLog(@"failed to open item %@ from iCloud", url);
                
            }
            
        }];
        
    }
    
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = @"Grocery list";
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            self.clearsSelectionOnViewWillAppear = NO;
            self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
        }
    }
    return self;
}
							
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _objects = [[NSMutableArray alloc] init];

    self.navigationItem.leftBarButtonItem = self.editButtonItem;

    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                                                                               target:self
                                                                               action:@selector(insertNewObject:)];
    self.navigationItem.rightBarButtonItem = addButton;
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(noteUpdated:)
                                                 name:groceryItemUpdated
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(loadGrocery)
                                                 name:UIApplicationDidBecomeActiveNotification
                                               object:nil];
    
}


- (void) noteUpdated:(id)sender {

    SMGroceryItem *i = (SMGroceryItem*)sender;

    NSUInteger index = [_objects indexOfObject:i];
    
    NSIndexPath *a = [NSIndexPath indexPathForItem:index inSection:0];
    
    //just reload reload that item
    [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:a]
                          withRowAnimation:UITableViewRowAnimationNone];
    
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (void)insertNewObject:(id)sender
{
    if (!_objects) {
        _objects = [[NSMutableArray alloc] init];
    }
    
    SMGroceryItem *groceryItem = [SMGroceryItem createNewItem];
    
    [groceryItem saveToURL:[groceryItem fileURL]
          forSaveOperation:UIDocumentSaveForCreating
         completionHandler:^(BOOL success) {
             
             if (success) {
                 
                 NSLog(@"new item created");
                 
                 // Opening the new item
                 [groceryItem openWithCompletionHandler:^(BOOL success) {
                     
                     NSLog(@"New item opened");
                     [self.tableView reloadData];
                     
                 }];
                 
             }
         }];
    
    [_objects addObject:groceryItem];

    
    
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objects.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView
                             dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {  
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                      reuseIdentifier:CellIdentifier];
        
        if ([[UIDevice currentDevice] userInterfaceIdiom] ==
            UIUserInterfaceIdiomPhone) {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
    }
   
    SMGroceryItem *groceryItem = [_objects objectAtIndex:indexPath.row];
    cell.textLabel.text = groceryItem.itemName;
   
    switch (groceryItem.documentState) {
    
        case UIDocumentStateClosed:
            NSLog(@"Closed");
            break;
            
       case UIDocumentStateNormal:
           NSLog(@"Normal");
           cell.textLabel.textColor = [UIColor blackColor];
            cell.userInteractionEnabled = YES;
           break;
            
       case UIDocumentStateInConflict:
            cell.textLabel.textColor = [UIColor redColor];
            cell.userInteractionEnabled = YES;
            NSLog(@"Conflict");
           break;
            
       case UIDocumentStateEditingDisabled:
            cell.textLabel.textColor = [UIColor yellowColor];
            cell.userInteractionEnabled = NO;
            NSLog(@"Editing disabled");
           break;
            
       case UIDocumentStateSavingError:
            NSLog(@"Saving error");
            break;
            
       default:
            NSLog(@"Unknown state");
            break;
   }
    return cell;
}
    


- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}


- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        SMGroceryItem *item = _objects[indexPath.row];
        
        [_objects removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
        
        NSError *err;
        [[NSFileManager defaultManager] removeItemAtURL:[item fileURL]
                                                  error:&err];
        
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {

    }
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    SMGroceryItem *object = [_objects objectAtIndex:indexPath.row];
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {

	    self.detailViewController = [[SMDetailViewController alloc] initWithNibName:@"SMDetailViewController_iPhone" bundle:nil];
	    self.detailViewController.detailItem = object;
        [self.navigationController pushViewController:self.detailViewController animated:YES];

    } else {
	
        self.detailViewController.detailItem = object;

    }
}

@end
