/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMDetailViewController.m
//  Grocery
//
//  Created by Cesare Rocchi on 12/17/12.
//  Copyright (c) 2012 Cesare Rocchi. All rights reserved.
//

#import "SMDetailViewController.h"

@interface SMDetailViewController ()
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
- (void)configureTags;
@end

@implementation SMDetailViewController

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;        
        [self configureView];
    }

    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }        
}

- (void)configureView
{
    if (self.detailItem) {
        self.itemNameField.text = self.detailItem.itemName;
        [self configureTags];
    }
}

- (void) configureTags {

    NSArray *tagsArray = [self.detailItem.tags allObjects];
    NSMutableArray *tagNames =
    [NSMutableArray arrayWithCapacity:tagsArray.count];
    for (SMTag *t in tagsArray) {
        [tagNames addObject:t.tagName];
    }
    NSString *s = [tagNames componentsJoinedByString:@","];
    self.tagsLabel.text = s;
    
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.tagsLabel.backgroundColor = [UIColor lightGrayColor];
    self.tagsLabel.userInteractionEnabled = YES;
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc]
                                          initWithTarget:self
                                          action:@selector(tagsTapped)];
    [self.tagsLabel addGestureRecognizer:tapGesture];
    [self configureView];
    
    UIBarButtonItem *saveButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSave target:self action:@selector(save)];
    
    self.navigationItem.rightBarButtonItem = saveButton;
}

- (void) viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];
    [self configureTags];
    
}

- (void) save {
    
    self.detailItem.itemName = self.itemNameField.text;
    NSManagedObjectContext *context = self.detailItem.managedObjectContext;

    NSError *error = nil;
    if (![context save:&error]) {
        NSLog(@"Error in saving item %@, %@", error, [error userInfo]);
        abort();
    }
    
}

- (void) tagsTapped {

    NSLog(@"tags tapped");
    
    SMTagPicker *picker;
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
    
        picker = [[SMTagPicker alloc] initWithNibName:@"SMTagPicker_iPhone" bundle:nil];
        
    } else {
    
        picker = [[SMTagPicker alloc] initWithNibName:@"SMTagPicker_iPad" bundle:nil];
        
    }

    picker.currentItem = self.detailItem;

    [self.navigationController pushViewController:picker
                                         animated:YES];

}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = NSLocalizedString(@"Item", @"Detail");
    }
    return self;
}
							
#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Master", @"Master");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
