/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMTagPicker.m
//  Grocery
//
//  Created by Cesare Rocchi on 12/17/12.
//  Copyright (c) 2012 Cesare Rocchi. All rights reserved.
//

#import "SMTagPicker.h"

@interface SMTagPicker ()

@end

@implementation SMTagPicker


- (void)viewDidLoad
{
    [super viewDidLoad];
    self.pickedTags = [[NSMutableSet alloc] init];
    self.title = @"Tag your item";

    NSError *error;
    if (![self.fetchedResultsController performFetch:&error]) {
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        
	}
    
    if (self.fetchedResultsController.fetchedObjects.count == 0) {
        
            
        SMTag *groceryStore = [NSEntityDescription
                    insertNewObjectForEntityForName:@"SMTag"
                    inManagedObjectContext:
                    self.currentItem.managedObjectContext];
        groceryStore.tagName =  @"grocery store";
        
        SMTag *greenGrocer = [NSEntityDescription
                            insertNewObjectForEntityForName:@"SMTag"
                            inManagedObjectContext:
                            self.currentItem.managedObjectContext];
        greenGrocer.tagName =  @"greengrocer";
        
        SMTag *mall = [NSEntityDescription
                              insertNewObjectForEntityForName:@"SMTag"
                              inManagedObjectContext:
                              self.currentItem.managedObjectContext];
        mall.tagName =  @"mall";
        
        NSError *error = nil;
        if (![self.currentItem.managedObjectContext save:&error]) {
            NSLog(@"Unresolved error %@, %@", error,
                  [error userInfo]);
            
        } else {

            NSLog(@"new tags added");
            [self.fetchedResultsController performFetch:&error];
        }
    }
    
    for (SMTag *tag in self.currentItem.tags) {
        
        [self.pickedTags addObject:tag];
        
    }

    
}

- (void) viewWillDisappear:(BOOL)animated {
    
    [super viewWillDisappear:animated];
    self.currentItem.tags = self.pickedTags;
    
    NSError *error = nil;
    if (![self.currentItem.managedObjectContext save:&error]) {
        
        NSLog(@"Core data error %@, %@", error, [error userInfo]);
        
        
    }
    
}

#pragma mark - Fetch result controller

- (NSFetchedResultsController *)fetchedResultsController
{
    
    if (_fetchedResultsController != nil) {
        return _fetchedResultsController;
    }
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    
    NSEntityDescription *entity = [NSEntityDescription
                                   entityForName:@"SMTag"
                                   inManagedObjectContext:
                                   self.currentItem.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc]
                                        initWithKey:@"tagName"
                                        ascending:NO];
    NSArray *sortDescriptors =
    [NSArray arrayWithObjects:sortDescriptor, nil];
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    NSFetchedResultsController *aFetchedResultsController =
	[[NSFetchedResultsController alloc]
     initWithFetchRequest:fetchRequest
     managedObjectContext:self.currentItem.managedObjectContext
     sectionNameKeyPath:nil
     cacheName:@"Master"];
    
    self.fetchedResultsController = aFetchedResultsController;
    
	NSError *error = nil;
    if (![self.fetchedResultsController performFetch:&error]) {
	    NSLog(@"Core data error %@, %@", error, [error userInfo]);
        
	}
    
    return _fetchedResultsController;
    
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{

    id <NSFetchedResultsSectionInfo> sectionInfo =
    [[self.fetchedResultsController sections] objectAtIndex:section];
    return [sectionInfo numberOfObjects];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        
    }
    
    cell.accessoryType = UITableViewCellAccessoryNone;
    
    SMTag *tag = (SMTag *)[self.fetchedResultsController
                           objectAtIndexPath:indexPath];
    if ([self.pickedTags containsObject:tag]) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }
    
    cell.textLabel.text = tag.tagName;
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{

    SMTag *tag = (SMTag *)[self.fetchedResultsController
                           objectAtIndexPath:indexPath];
    UITableViewCell * cell = [self.tableView
                              cellForRowAtIndexPath:indexPath];
    [cell setSelected:NO animated:YES];
    
    if ([self.pickedTags containsObject:tag]) {
        
        [self.pickedTags removeObject:tag];
        cell.accessoryType = UITableViewCellAccessoryNone;
        
    } else {
        
        [self.pickedTags addObject:tag];
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
        
    }
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
